import matplotlib.pyplot as plt
import matplotlib
import datetime as datetime

from datetime import timedelta
import numpy as np
import pickle
import bisect
from matplotlib.backends.backend_pdf import PdfPages


def SegmentData(T,V):
    # We first identify gaps between data to avoid average data belonging to different segments
    tGap = []
    for i in range(len(T)-1):
        tGap.append( abs((T[i+1] - T[i]).total_seconds() ))
    
    sortedIDs = [x for _,x in sorted(zip(tGap, range(len(T)-1)))]
    tGap.sort()
    #sortedIDs.reverse()
    
    n = bisect.bisect(tGap, 60)
    
    sortedIDs = sortedIDs[n:]
    sortedIDs.append(len(T)-1)
    
    sortedIDs.sort()
    
    sortedIDs.insert(0,-1)
    
    # sortedIDs are the IDs of the end of the segments
    
    t = []
    v = []
    
    for i in  range(len(sortedIDs)-1):
        
        m0 = sortedIDs[i] + 1
        m1 = sortedIDs[i + 1]
        
        
        t.append ([])
        v.append ([])
        t[-1].append(T[m0])
        v[-1].append(V[m0])

        t0 = T[m0]
        i0 = m0
        for j in range(m0+1, m1+1):
            if (T[j]-t0).total_seconds() >= 3600:
                t[-1].append(T[i0] + timedelta(seconds=1800))
                v[-1].append(np.average(V[i0:j]))
                i0 = j
                t0 = T[j]

        t[-1].append(T[m1])
        v[-1].append(V[m1])

    return (t, v)


str2date = lambda x: datetime.datetime.strptime(x.decode("utf-8"), '%m/%d/%y %H:%M:%S')

roots = ['InjectionPressure', 'InjectionRate','P_Interval_Flow','P_Bottom_Flow', 'PDT_Flow', 'PST_Flow', 'InjectionIntervalTemp','P_Bottom_Temp_Questionable','P_Bottom_Temp_Upgraded','P_Interval_Temp_Questionable','P_Interval_Temp_Upgraded']
AllData = {}

if True:
    for root in roots:
        print (root)
        myTime = np.genfromtxt(root+'.csv', dtype=None, usecols=0, delimiter=',', converters = {0: str2date}, skip_header=1)
        myV = np.genfromtxt(root+'.csv', dtype=None, usecols=1, delimiter=',', skip_header=1)
        AllData[root] = [myTime, myV]
        pickleFile = open('AllData.pickle', "wb")
        pickle.dump(AllData, pickleFile)
        pickleFile.close()
        exit()
else:
    pickleFile = open('AllData.pickle', "rb")
    AllData= pickle.load(pickleFile)
    pickleFile.close()
    print ('Finished Loading')


fig, ax = plt.subplots(nrows=4, ncols=1, figsize=(11, 9), facecolor='w', edgecolor='k') #

a = ax[0]
root = 'InjectionRate'
t, v = SegmentData(AllData[root][0], AllData[root][1])
for i in range(len(t)):
    if i == 0:
        a.plot(t[i], v[i], 'g', label='Injection Rate')
    else:
        a.plot(t[i], v[i], 'g')
a.legend(loc="upper right", ncol=1)

a = ax[1]
root = 'InjectionPressure'
t, v = SegmentData(AllData[root][0], AllData[root][1])
for i in range(len(t)):
    if i == 0:
        a.plot(t[i], np.array(v[i])*0.00689476, 'b', label='Injection Pressure')
    else:
        a.plot(t[i], np.array(v[i])*0.00689476, 'b')
a.legend(loc="upper right", ncol=1)


a = ax[2]
roots = ['P_Interval_Flow','P_Bottom_Flow', 'PDT_Flow', 'PST_Flow', 'TotalLeak']
colors = ['r', 'darkred', 'lime', 'magenta', 'gray']
labels = ['E1-PI Flow','E1-PB Flow', 'E1-PDT Flow', 'E1-PST Flow', 'Total Leak Rate']
# The calculation of total leak assume all the four channels are synchronized.
AllData ['TotalLeak'] = [] 
AllData ['TotalLeak'].append(AllData['P_Interval_Flow'][0])
AllData ['TotalLeak'].append( AllData['P_Interval_Flow'][1] + AllData['P_Bottom_Flow'][1] + AllData['PDT_Flow'][1] + AllData['PST_Flow'][1])


for i in range(len(roots)):
    root = roots[i]
    color = colors[i]
    a.plot([],[], c=color, label = labels[i])
    t, v = SegmentData(AllData[root][0], AllData[root][1])
    for j in range(len(t)):
        if len(t[j] )> 1:
            print (root, t[j][0], t[j][-1])
            if  t[j][0] > datetime.datetime(2019,12,17,15,0,0) and t[j][0] < datetime.datetime(2020,1,15,17,30,0) and root[:2] == 'P_':
                ls = 'dotted'
            else:
                ls = 'solid'
            if j == 0:
                a.plot(t[j], v[j], c=color, ls = ls)
            else:
                a.plot(t[j], v[j], c=color, ls = ls)
a.plot([],[], c='gray', ls='dotted', label = 'Questionable Data')
a.legend(loc="upper right", ncol=6)

# 
a = ax[3]
roots = ['InjectionIntervalTemp','P_Interval_Temp_Questionable','P_Interval_Temp_Upgraded','P_Bottom_Temp_Questionable','P_Bottom_Temp_Upgraded']
labels = ['Injection Temp.', '','E1-PI Temp.','','E1-PB Temp.']
colors = ['k', 'r', 'r', 'darkred', 'darkred']
lss = ['solid', 'dotted', 'solid', 'dotted', 'solid']
t, v = SegmentData(AllData[root][0], AllData[root][1])
for i in range(len(roots)):
    root = roots[i]
    label = labels[i]
    color = colors[i]
    ls = lss[i]
    t, v = SegmentData(AllData[root][0], AllData[root][1])
    for j in range(len(t)):
        if j == 0 and labels[i] != '':
            a.plot(t[j], v[j], c=color, label = label, ls = ls)
        else:
            a.plot(t[j], v[j], c=color, ls = ls)
a.plot([],[], c='gray', ls='dotted', label = 'Questionable Data')
a.legend(loc="upper right", ncol=4)


ylims = [[0,0.6], [0,50], [0,0.6], [10, 45]]

titles = ['Injection Rate (L/min)', 'Injection Pressure (MPa)', 'Out-Flow rate (L/min)', 'Temperature (°C)'] 
for i in range(len(ax)):
    a = ax[i]
    a.set_xlim(datetime.datetime(2019,3, 15), datetime.datetime(2020,2,15))
    a.set_ylim(ylims[i])
    a.set_ylabel(titles[i])
    a.tick_params(direction = 'in')
#     if i < 3:
#         a.set_xticklabels([])
        
    a.grid()
    
plt.tight_layout()
#plt.savefig('SummaryPlot.png', dpi = 250)
plt.show()
