import matplotlib.pyplot as plt
import matplotlib
import datetime as dt
from datetime import timedelta
import numpy as np
import pickle
import bisect
from matplotlib.backends.backend_pdf import PdfPages

def MapChannelsInFile (filename):
    hasChannels = []
    idChannels = []
    file = open(filename, 'r')
    file.readline()
    myChannels = file.readline()[:-1].split(',')
    for channel in AllChannels:
        if channel in myChannels:
            hasChannels.append(channel)
            idChannels.append(myChannels.index(channel))
    file.close()
    return (hasChannels, idChannels)

def ReadTime(filename):
    myTimes = []
    file = open(filename)
    for i in range(3):
        file.readline()
    for line in file:
        lineTime = dt.datetime.strptime(line.split(',')[0], '%m/%d/%y %H:%M:%S')  #02/04/20 16:29:29
        myTimes.append(lineTime)
    file.close()
    return (myTimes)

ChannelMaps = {}
myColors = ['k', 'red', 'maroon', 'blue', 'magenta', 'seagreen', 'yellowgreen', 'gray', 'darkviolet', 'chocolate', 'deeppink', 'violet', 'orange', 'lime', 'tan', 'forestgreen']

PChannels = ['SNL 10','Air P', 'LBNL 03', 'LBNL 05', 'LBNL 07', 'PNNL 14', 'SNL 06', 'SNL 07', 'SNL 08', 'SNL 09', 'SNL 11', 'SNL 17',]
TChannels = ['LBNL 02', 'LBNL 04', 'LBNL 08', 'PNNL 05', 'PNNL 06', 'PNNL 12t', 'PNNL 15t',]
QChannels = ['Quizix Flow', 'Triplex Net Flow','PNNL 13', 'I Bot Flow', 'I Collar Flow', 'OB Collar flow', 'OT Collar flow', 'OTB Collar flow', 'OTT Collar flow', 'P Bot Flow', 'P Collar Flow', 'PDB Collar Flow', 'PDT Collar Flow', 'PSB Collar Flow', 'PST Collar Flow', 'PNNL 16']
EChannels = ['PNNL 12c',  'PNNL 15c',]

ChannelMap = {'SNL 10':'Injection pressure', 
'Air P':'Air P', 
 'LBNL 03':'Production Interval Pressure', 
 'LBNL 05':'Injection Bottomhole Pressure', 
 'LBNL 07':'Production Bottomhole Pressure', 
 'PNNL 14':'Production Interval Pressure', 
 'SNL 06':'Injection Upper Packer Element Pressure', 
 'SNL 07':'Injection Lower Packer Element Pressure', 
 'SNL 08':'Production Upper Packer Element Pressure', 
 'SNL 09':'Production Lower Packer Element Pressure', 
 'SNL 11':'Injection Bottomhole Pressure', 
 'SNL 17':'Production Bottomhole Pressure ', 
 'LBNL 02':'Injection Interval Temperature', 
 'LBNL 04':'Production Interval Temperature', 
 'LBNL 08':'Production Bottomhole Temperature', 
 'PNNL 05':'Recirc Inlet Thermistor Temperature', 
 'PNNL 06':'Recirc Outlet Thermistor Temperature', 
 'PNNL 12t':'Water Tank Outlet Temperature', 
 'PNNL 15t':'Production Interval Outlet Temperature', 
 'Quizix Flow':'Quizix Flow', 
 'Triplex Net Flow':'Triplex Net Flow', 
 'PNNL 13':'Production Interval Flow', 
 'I Bot Flow':'', 
 'I Collar Flow':'', 
 'OB Collar flow':'', 
 'OT Collar flow':'', 
 'OTB Collar flow':'', 
 'OTT Collar flow':'', 
 'P Bot Flow':'', 
 'P Collar Flow':'', 
 'PDB Collar Flow':'', 
 'PDT Collar Flow':'', 
 'PSB Collar Flow':'', 
 'PST Collar Flow':'', 
 'PNNL 12c':'Water Tank Outlet Electrical Conductivity', 
 'PNNL 15c':'Production Interval Electrical Conductivity',
 'PNNL 16': 'Chiller Circulation Rate'
}
AllChannels = PChannels + TChannels + QChannels + EChannels

# 
# reloadData = True
# if reloadData:

csvFiles = []
listFiles = open('./RawFlowData/file_list.txt', 'r')
for line in listFiles:
    csvFiles.append( line[:-1] )   # The -1 part remove the line break
print (csvFiles)
listFiles.close()

# Sort the files by time

time0File = []
time1File = []

for filename in csvFiles:
    file = open('./RawFlowData/'+filename)
    for i in range(3):
        file.readline()
    firstLine = file.readline()
    print (filename, firstLine.split(',')[0])
    lineTime = dt.datetime.strptime(firstLine.split(',')[0], '%m/%d/%y %H:%M:%S')  #02/04/20 16:29:29
    time0File.append(lineTime)
    
    for line in file:
        pass
    lineTime = dt.datetime.strptime(line.split(',')[0], '%m/%d/%y %H:%M:%S')  #last line
    time1File.append(lineTime)
    
    file.close()
    
# Sort the files.  The original naming was not very disciplined
csvFiles = [x for _,x in sorted(zip(time0File,csvFiles))]
time1File = [x for _,x in sorted(zip(time0File,time1File))]
time0File.sort()


AllData = {'timeWindow':[time0File[0], time1File[-1]]}

for filename in csvFiles:
    print ('Reading file: '+ filename)
    channels, cols = MapChannelsInFile ('./RawFlowData/'+filename)
    fileData = np.loadtxt('./RawFlowData/'+filename, delimiter=',', usecols=cols, unpack = True, skiprows=3)
    fileTimes = ReadTime('./RawFlowData/'+filename)
    count = 0
    for channel in channels:
        if not channel in AllData:
            AllData[channel] = [[],[]]
        AllData[channel][0] += fileTimes
        AllData[channel][1] += list(fileData[count])
        count += 1
# Filter flow data using moving average.  Flow measurements were noisy
print ('Filtering flow channels using moving average')
FlowDataMV = {}
for channel in QChannels:
    if channel in AllData:
        FlowDataMV[channel] = [[],[]]
        nWin = 599.99 # seconds
        count = 0
        Q = 0.0
        for i in range(len(AllData[channel][0])):
            if count == 0:
                refTime = AllData[channel][0][i]
            Q += AllData[channel][1][i]
            count += 1
            if (AllData[channel][0][i] - refTime).seconds >= nWin or i == len(AllData[channel][0])-1 :
                FlowDataMV[channel][1].append(Q/count)
                FlowDataMV[channel][0].append(AllData[channel][0][i-int(count/2)])
                count = 0
                Q = 0.0
            


# Find the Saturday before the first data point
window = [AllData['timeWindow'][0] + dt.timedelta(days=-AllData['timeWindow'][0].weekday()-2, hours = -AllData['timeWindow'][0].hour, minutes = -AllData['timeWindow'][0].minute, seconds = -AllData['timeWindow'][0].second)]
#window[0] = dt.datetime(2020,4,28,0,0,0)
window.append(window[0]+timedelta(days=30))

groupedChannels = [QChannels[:-1], QChannels[:-1], PChannels, TChannels, EChannels]  # The last channel in QChannels is chiller circulation channel. The value is high and we will put it in a twinx as a special treatment
groupLabels = ['Flow rates (L/min)',  'Flow rates, filtered (L/min)', 'Pressure (psi)', 'Temperature (°C)', 'EC (mS/cm)']
with PdfPages('LabeledWeekly_CompleteFiles.pdf') as pdf:

    while window[0] < AllData['timeWindow'][1]:
        print (window)
        
        fig, axes = plt.subplots(nrows=5, ncols=1, figsize=(20, 16), facecolor='w', edgecolor='k', dpi=250) #
        
        for i in range(len(groupedChannels)):
            print ('plotting ', groupLabels[i])
            j = 0
            for channel in groupedChannels[i]:
                if i == 1:
                    if channel in FlowDataMV:
                        myLeft, myRight = bisect.bisect_left(FlowDataMV[channel][0], window[0]), bisect.bisect_right(FlowDataMV[channel][0], window[1])
                        axes[i].plot(FlowDataMV[channel][0][myLeft:myRight], FlowDataMV[channel][1][myLeft:myRight], label =channel+': '+ChannelMap[channel], c=myColors[j], lw=0.8, rasterized = True)
                        j += 1
                else:
                    if channel in AllData:
                        myLeft, myRight = bisect.bisect_left(AllData[channel][0], window[0]), bisect.bisect_right(AllData[channel][0], window[1])
                        axes[i].plot(AllData[channel][0][myLeft:myRight], AllData[channel][1][myLeft:myRight], label =channel+': '+ChannelMap[channel], c=myColors[j], lw=0.8, rasterized = True)
                        j += 1
            axes[i].set_xlim(window)
            axes[i].set_ylabel(groupLabels[i])
            
            if i != 1:
                leg = axes[i].legend(loc='upper left', bbox_to_anchor=(1.04, 1.05), ncol=1)
                for legobj in leg.legendHandles:
                    legobj.set_linewidth(2.0)
            axes[i].grid()

        axChiCir = axes[0].twinx()
        channel = QChannels[-1]
        myLeft, myRight = bisect.bisect_left(AllData[channel][0], window[0]), bisect.bisect_right(AllData[channel][0], window[1])
        axChiCir.plot(AllData[channel][0][myLeft:myRight], AllData[channel][1][myLeft:myRight], label =channel+': '+ChannelMap[channel], c=myColors[-1], lw=0.8, rasterized = True)
        axChiCir.set_xlabel('Chiller circulation rate (L/m)')
        axChiCir.set_xlim(window)
        leg = axChiCir.legend(loc='lower left', bbox_to_anchor=(1.04, 1.06), ncol=1)
        for legobj in leg.legendHandles:
            legobj.set_linewidth(2.0)
            
        axes[0].set_ylim([0,0.5])
        axes[1].set_ylim([0,0.1])
        axChiCir.set_ylim([-10,40])
        axes[3].set_ylim([0,40])

        plt.tight_layout()
#         plt.savefig('PNGs/'+str(window[0].date())+'.png', dpi=120)
#         exit()
    #     plt.close(fig='all')
        pdf.savefig()
        plt.close()
#        exit()
        window[0] += timedelta(days=28)
        window[1] += timedelta(days=28)
        
