import matplotlib.pyplot as plt
import matplotlib
import datetime as datetime

from datetime import timedelta
import numpy as np
import pickle
import bisect
from matplotlib.backends.backend_pdf import PdfPages


def SegmentData(T,V):
    tGap = []
    for i in range(len(T)-1):
        tGap.append( abs((T[i+1] - T[i]).total_seconds() ))
    
    sortedIDs = [x for _,x in sorted(zip(tGap, range(len(T)-1)))]
    tGap.sort()
    #sortedIDs.reverse()
    
    n = bisect.bisect(tGap, 60)
    
    sortedIDs = sortedIDs[n:]
    sortedIDs.sort()
    
    sortedIDs.append(len(T)-1)
   
    sortedIDs.insert(0 , -1)
    
    t = []
    v = []
    
    w = 360 # Moveing average window.  360 is one hour as the data recording interval was 10s
    
    for i in  range(len(sortedIDs)-1):
        t.append ([])
        v.append ([])
#         t[-1].append(T[sortedIDs[i]])
#         v[-1].append(V[sortedIDs[i]])

        if sortedIDs[i+1] - sortedIDs[i] > w*2:
            for j in range(sortedIDs[i]+w//2, sortedIDs[i+1], w):
                t[-1].append(T[j])
                v[-1].append(np.average(V[j-w//2: j+w-w//2]))
#         t[-1].append(T[sortedIDs[i+1]-1])
#         v[-1].append(V[sortedIDs[i+1]-1])

    return (t, v)


str2date = lambda x: datetime.datetime.strptime(x.decode("utf-8"), '%m/%d/%y %H:%M:%S')


roots = ['InjectionPressure', 'InjectionRate','P_Interval_Flow','P_Bottom_Flow', 'PDT_Flow', 'PST_Flow', 'InjectionIntervalTemp','P_Bottom_Temp_Questionable','P_Bottom_Temp_Upgraded','P_Interval_Temp_Questionable','P_Interval_Temp_Upgraded']


if False:
    AllData = {}
    for root in roots:
        print (root)
        myTime = np.genfromtxt(root+'.csv', dtype=None, usecols=0, delimiter=',', converters = {0: str2date}, skip_header=0)
        myV = np.genfromtxt(root+'.csv', dtype=None, usecols=1, delimiter=',')
        AllData[root] = [myTime, myV]
    pickleFile = open('AllData.pickle', "wb")
    pickle.dump(AllData, pickleFile)
    pickleFile.close()
    exit()
else:
    pickleFile = open('AllData.pickle', "rb")
    AllData= pickle.load(pickleFile)
    pickleFile.close()
    print ('Finished Loading')




fig, ax = plt.subplots(nrows=2, ncols=2, figsize=(11, 6), facecolor='w', edgecolor='k') #

a = ax[0][0]
root = 'InjectionRate'
t, v = SegmentData(AllData[root][0], AllData[root][1])
for i in range(len(t)):
    if i == 0:
        a.plot(t[i], v[i], 'g', label='Injection Rate')
    else:
        a.plot(t[i], v[i], 'g')

a = ax[0][1]
root = 'InjectionRate'
t, v = SegmentData(AllData[root][0], AllData[root][1])
for i in range(len(t)):
    if i == 0:
        a.plot(t[i], v[i], 'g', label='Injection Rate')
    else:
        a.plot(t[i], v[i], 'g')
a.legend(loc="upper right", ncol=1)


# 
a = ax[1][0]
roots = ['P_Interval_Temp_Questionable','P_Bottom_Temp_Questionable']
labels = ['E1-PI Temp.', 'E1-PB Temp.']
colors = ['r', 'darkred']
for i in range(len(roots)):
    root = roots[i]
    label = labels[i]
    color = colors[i]
    t, v = SegmentData(AllData[root][0], AllData[root][1])
    for j in range(len(t)):
        if j == 0 :
            a.plot(t[j], v[j], c=color, label = label)
        else:
            a.plot(t[j], v[j], c=color, ls='dotted')
a = ax[1][1]
roots = ['P_Interval_Temp_Upgraded','P_Bottom_Temp_Upgraded']
labels = [ 'E1-PI Temp.', 'E1-PB Temp.']
colors = ['r', 'darkred']
for i in range(len(roots)):
    root = roots[i]
    label = labels[i]
    color = colors[i]
    t, v = SegmentData(AllData[root][0], AllData[root][1])
    for j in range(len(t)):
        if j == 0 :
            a.plot(t[j], v[j], c=color, label = label)
        else:
            a.plot(t[j], v[j], c=color)
a.plot([],[], c='gray', ls='dotted', label = 'Questionable Data')
a.legend(loc="upper right", ncol=2)


ax[0][0].set_ylim(0, 0.6)
ax[0][1].set_ylim(0, 0.6)
ax[1][0].set_ylim(25, 35)
ax[1][1].set_ylim(25, 35)
ax[0][0].set_xlim(datetime.datetime(2019,4, 1), datetime.datetime(2019,5,21))
ax[1][0].set_xlim(datetime.datetime(2019,4, 1), datetime.datetime(2019,5,21))
ax[0][1].set_xlim(datetime.datetime(2019,12,1), datetime.datetime(2020,1,1))
ax[1][1].set_xlim(datetime.datetime(2019,12, 1), datetime.datetime(2020,1,1))

ax[0][1].set_yticklabels([])
ax[1][1].set_yticklabels([])
ax[0][0].set_ylabel('Injection Rate (L/min)')
ax[1][0].set_ylabel('Temperature (°C)')


for aa in ax:
    for a in aa:
        a.get_xaxis().set_major_locator(matplotlib.ticker.MultipleLocator(10))
        a.grid()
        a.tick_params(direction = 'in')
    
#ax.set_ylim(0,0.5)
plt.tight_layout()
plt.savefig('TemperatureAnomaly.png', dpi = 250)
plt.show()
